package net.minecraft.thKaguyaMod;

import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;
import net.minecraft.*;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.MathHelper;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;
import java.util.Random;

public class thShotLib
{
	//弾幕関連の汎用関数群
	
	//弾の色
	public static final int RED    = 0;
	public static final int BLUE   = 1;
	public static final int GREEN  = 2;
	public static final int YELLOW = 3;
	public static final int PURPLE = 4;
	public static final int AQUA   = 5;
	public static final int ORANGE = 6;
	public static final int WHITE  = 7;
	
	//弾形状
	public static final int SMALL[]     = {  0,  1,  2,  3,  4,  5,  6,  7};
	public static final int TINY[]      = {  8,  9, 10, 11, 12, 13, 14, 15};
	public static final int MEDIUM[]    = { 16, 17, 18, 19, 20, 21, 22, 23};
	public static final int PEARL[]     = { 24, 25, 26, 27, 28, 29, 30, 31};
	public static final int CIRCLE[]    = { 32, 33, 34, 35, 36, 37, 38, 39};
	public static final int LIGHT[]     = { 40, 41, 42, 43, 44, 45, 46, 47};
	public static final int SCALE[]     = { 48, 49, 50, 51, 52, 53, 54, 55};
	public static final int BUTTERFLY[] = { 56, 57, 58, 59, 60, 61, 62, 63};
	public static final int SMALLSTAR[] = { 64, 65, 66, 67, 68, 69, 70, 71};
	public static final int STAR[]      = { 72, 73, 74, 75, 76, 77, 78, 79};
	public static final int RICE[]		= { 80, 81, 82, 83, 84, 85, 86, 87};
	public static final int KNIFE[]     = {224,225,226,227,228,229,230,231};
	public static final int WIND[]		= {232,233,234,235,236,237,238,239};
	public static final int BIG[]       = {240,241,242,243,244,245,246,247};
	public static final int KISHITU[]   = {248,249,250,251,252,253,254,255};
	
	//弾の色の名前
	public static final String COLOR_NAME_JP[]	={
		"赤", "青", "緑", "黄", "紫", "水色", "橙", "白", "虹色", "ランダム", "暖色", "寒色", "", "", "", ""};
	//弾の名前
	public static final String SHOT_NAME_JP[]	={
		  "小弾",  "粒弾",  "中弾","真珠弾",  "輪弾",  "光弾",  "鱗弾",  "蝶弾",
		"小星弾",  "星弾",  "米弾","未定義","未定義","未定義","未定義","未定義",
		"未定義","未定義","未定義","未定義","未定義","未定義","未定義","未定義",
		"未定義","未定義","未定義","未定義","ナイフ",  "風弾",  "大弾","気質弾"};
	
	//形状ごとの弾のダメージを定義
	public static final int DAMAGE[] = {2, 1, 3, 5, 2, 3, 2, 3,
										2, 3, 0, 0, 0, 0, 0, 0,
										0, 0, 0, 0, 0, 0, 0, 0,
										0, 0, 0, 0, 3, 4, 5, 5};
	
	//形状ごとの弾の大きさを定義
	public static final float SIZE[] = {0.30F, 0.15F, 0.60F, 0.30F, 0.30F, 0.30F, 0.30F, 0.60F, 
										0.15F, 0.30F, 0.00F, 0.00F, 0.00F, 0.00F, 0.00F, 0.00F,
										0.00F, 0.00F, 0.00F, 0.00F, 0.00F, 0.00F, 0.00F, 0.00F,
										0.00F, 0.00F, 0.00F, 0.00F, 0.30F, 1.00F, 1.20F, 0.50F};
	
	//特殊弾の定義
	public static final int FALL01          =  30;
	public static final int BOUND01         =  40;
	public static final int BOUNDFALL01     =  50;
	public static final int FREEZE01        = 160;
	public static final int KAPPA01         = 170;
	public static final int WIND01          = 180;
	public static final int KISHITU01		= 190;
	public static final int FLOWER01		= 200;
	public static final int STARDUST01      = 210;
	public static final int KOKUSHI01		= 230;
	public static final int AJA01           = 240;
	
	
	/*
	弾生成処理メソッド、命名規則
	createShot○○ ○○は数字が二桁入り、一の桁を見た時
	○1 最も基礎的で、弾番号を指定しただけでダメージや大きさが指定されるもの
	○2 1のものに速度変化を指定できるもの
	○3 2に加え、発射地点を指定できるもの
	十の桁を見た時
	0○ 進行方向を角度で指定するもの
	1○ 進行方向をベクトルで指定するもの
	4○ 特殊弾を指定できる0○
	*/
	
	/*
	すべての弾の基礎となる生成処理
	entityUser   : 弾の持ち主、当たった場合持ち主が当てたことになる。また、持ち主が死ぬと消滅したり、アイテム化する
	entitySource : 弾の発射元。あまり意味は無い
	xPos         : 弾のX座標の発射地点
	yPos         :
	zPos         :
	xVector      : 弾のX方向のベクトル値
	yVector      :
	zVector      :
	firstSpeed   : 弾の初期速度
	maxSpeed     : 弾の最高速度。これ以上は速くならない
	addSpeed     : 弾の加速度。毎フレームこの数値分速くなる
	xVectorG     : 弾のX軸方向の重力値。0.0Dで重力の影響を受けない
	yVectorG     :
	zVectorG     :
	damage       : 弾のダメージ
	form         : 弾の形状。小弾、大弾、星弾、光弾などはこれできまる。また色も決まる。thShotLib.BUTTERFLY[ thShotLib.YELLOW ]というように指定できる
	size         : 弾の大きさ。当たり判定、描画サイズに関わる
	endTime      : 弾の消滅時間。この時間になったら強制的に消滅する。懐中時計で止められている場合は時間は進まず停滞する
	special      : 弾の特殊な処理を指定する。普通の弾は0。FALL01とすれば落下弾になる
	*/
	public static final EntityTHShot createShotBaseA(EntityLivingBase entityUser, Entity entitySource, double xPos, double yPos, double zPos,
											double xVector, double yVector, double zVector, double firstSpeed, double maxSpeed, double addSpeed,
											double xVectorG, double yVectorG, double zVectorG,
											int damage, int form, float size, int endTime, int special)
	{
		EntityTHShot entityTHShot;
		entityTHShot = new EntityTHShot(entitySource.worldObj, entityUser, entitySource, xPos, yPos, zPos,
										xVector, yVector, zVector, firstSpeed, maxSpeed, addSpeed,
										xVectorG, yVectorG, zVectorG,
										damage, form, size, endTime, special);
		if(!entitySource.worldObj.isRemote)
		{
			entitySource.worldObj.spawnEntityInWorld(entityTHShot);
			return entityTHShot;
		}
		return null;
	}
	
	/*
	発射方向を角度で指定できるすべての弾の基礎となる生成処理
	entityUser   : 弾の持ち主、当たった場合持ち主が当てたことになる。また、持ち主が死ぬと消滅したり、アイテム化する
	entitySource : 弾の発射元。あまり意味は無い
	xPos         : 弾のX座標の発射地点
	yPos         :
	zPos         :
	angleXZ      : 水平方向の角度
	angleY       : 垂直方向の角度
	firstSpeed   : 弾の初期速度
	maxSpeed     : 弾の最高速度。これ以上は速くならない
	addSpeed     : 弾の加速度。毎フレームこの数値分速くなる
	xVectorG     : 弾のX軸方向の重力値。0.0Dで重力の影響を受けない
	yVectorG     :
	zVectorG     :
	damage       : 弾のダメージ
	form         : 弾の形状。小弾、大弾、星弾、光弾などはこれできまる。また色も決まる。thShotLib.BUTTERFLY[ thShotLib.YELLOW ]というように指定できる
	size         : 弾の大きさ。当たり判定、描画サイズに関わる
	endTime      : 弾の消滅時間。この時間になったら強制的に消滅する。懐中時計で止められている場合は時間は進まず停滞する
	special      : 弾の特殊な処理を指定する。普通の弾は0。FALL01とすれば落下弾になる
	*/
	public static final EntityTHShot createShotBaseB(EntityLivingBase user, Entity source, double xPos, double yPos, double zPos,
											float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed,
											double xVectorG, double yVectorG, double zVectorG,
											int damage, int form, float size, int endTime, int special)
	{
		float angleXZ_rad = radTrans(angleXZ);//水平角度をラジアンに変更
		float angleY_rad = radTrans(angleY);//垂直角度をラジアンに変更
		double xVector = -Math.sin(angleXZ_rad) * Math.cos(angleY_rad);
		double yVector = -Math.sin(angleY_rad);
		double zVector =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad);
		
		return createShotBaseA(	user, source, xPos, yPos, zPos, xVector, yVector, zVector,
						firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG, damage, form, size, endTime, special);
	}
	/*
	使用者が発射地点になる最も一般的な弾を生成
	/ user       : 発射地点であり、発射主になる。
	/ angleXZ    : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY     : 垂直方向の角度。同上
	/ speed      : 弾の速度。目安として、0.1Dでかなり遅い、1.0Dで速い。
	/ shotType   : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot01(EntityLivingBase user, float angleXZ, float angleY, double speed, int shotType)
	{
		return createShot02(user, angleXZ, angleY, speed, speed, 1.0D, shotType);
	}
	
	/*
	使用者が発射地点になる一般的な弾に加速度が付属したものを生成
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ speed        : 弾の初期速度。目安として、0.1Dでかなり遅い、1.0Dで速い。
	/ maxSpeed     : 最高速度、または最低速度。初期速度より速いなら加速し、遅いなら減速していく
	/ accelerate   : 加速度または減速度。毎フレームこの速度分だけ減ったり増えたりする
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot02(EntityLivingBase user, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int shotType)
	{	
		return createShot03(user, user.posX, user.posY, user.posZ, angleXZ, angleY, firstSpeed, maxSpeed, addSpeed, shotType);
	}
	
	/*
	一般的なショットに発射地点と加速度を指定できるもの
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ xPos         : 発射地点のX座標
	/ yPos         :
	/ zPos         :
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ speed        : 弾の初期速度。目安として、0.1Dでかなり遅い、1.0Dで速い。
	/ maxSpeed     : 最高速度、または最低速度。初期速度より速いなら加速し、遅いなら減速していく
	/ accelerate   : 加速度または減速度。毎フレームこの速度分だけ減ったり増えたりする
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot03(EntityLivingBase user, double xPos, double yPos, double zPos, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int shotType)
	{
		int shotForm = shotType / 8;//形状は8の倍数で変化する

		return createShotBaseB(user, user, xPos, yPos, zPos, angleXZ, angleY,
						firstSpeed, maxSpeed, addSpeed, 0.0D, 0.0D, 0.0D, DAMAGE[shotForm], shotType, SIZE[shotForm], 120, 0);
	}
	
	/*
	使用者が発射地点になる最も一般的な弾を生成
	/ user       : 発射地点であり、発射主になる。
	/ xVector    : 弾のX方向のベクトル値
	/ yVector    :
	/ zVector    :
	/ speed      : 弾の速度。目安として、0.1Dでかなり遅い、1.0Dで速い。
	/ shotType   : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot11(EntityLivingBase user, double xVector, double yVector, double zVector, double speed, int shotType)
	{
		int shotForm = shotType / 8;//形状は8の倍数で変化する
		
		return createShotBaseA(user, user, user.posX, user.posY, user.posZ, xVector, yVector, zVector,
						speed, speed, 1.0D, 0.0D, 0.0D, 0.0D, DAMAGE[shotForm], shotType, SIZE[shotForm], 120, 0);
	}
	
	/*
	使用者が発射地点になる一般的な弾に加速度が付属したものを生成
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ speed        : 弾の初期速度。目安として、0.1Dでかなり遅い、1.0Dで速い。
	/ maxSpeed     : 最高速度、または最低速度。初期速度より速いなら加速し、遅いなら減速していく
	/ accelerate   : 加速度または減速度。毎フレームこの速度分だけ減ったり増えたりする
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	/ special      : 特殊弾IDを指定
	*/
	public static final void createShot22(EntityLivingBase EntityLivingBase, float angleXZ, float angleY, double speed, double maxSpeed, double accelerate, int shotType, int special)
	{
		EntityTHShot entityTHShot;
		int shotForm = shotType / 8;//形状は8の倍数で変化する
		double vectorX, vectorY, vectorZ;
		//MathHelper.sinでは度ではなくラジアンを使用するため変換する
		float angleXZ_rad = radTrans(angleXZ);//水平角度をラジアンに変更
		float angleY_rad = radTrans(angleY);//垂直角度をラジアンに変更
		
		//２つの角度からベクトルを計算
		vectorX = -MathHelper.sin(angleXZ_rad) * MathHelper.cos(angleY_rad);
		vectorY = -MathHelper.sin(angleY_rad);
		vectorZ =  MathHelper.cos(angleXZ_rad) * MathHelper.cos(angleY_rad);
		
		//EntityTHShotに値を入れる
		entityTHShot = new EntityTHShot(EntityLivingBase.worldObj, EntityLivingBase, EntityLivingBase, 
			EntityLivingBase.posX, EntityLivingBase.posY, EntityLivingBase.posZ,
			vectorX, vectorY, vectorZ,
			speed, maxSpeed, accelerate,
			0.0D, 0.0D, 0.0D, DAMAGE[shotForm], shotType, SIZE[shotForm], 120, special);
		
		if(!EntityLivingBase.worldObj.isRemote)
		{
			//弾を生成する
			EntityLivingBase.worldObj.spawnEntityInWorld(entityTHShot);
		}
	}
	
	/*
	使用者が発射地点になる一般的な弾を生成の特殊弾を指定できるもの
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ speed        : 弾の速度。目安として、0.1Dでかなり遅い、1.0Dで速い。
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot41(EntityLivingBase user, float angleXZ, float angleY, double speed, int shotType, int special)
	{
		return createShot42(user, angleXZ, angleY, speed, speed, 1.0D, shotType, special);
	}
	
	/*
	使用者が発射地点になる一般的な弾を生成の特殊弾を指定できるものに加速度も指定できるもの
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ firstSpeed   : 弾の初期速度。
	/ maxSpeed     : 弾の最高速度
	/ addSpeed     : 弾の加速度
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot42(EntityLivingBase user, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int shotType, int special)
	{
		return createShot43(user, user.posX, user.posY, user.posZ, angleXZ, angleY, firstSpeed, maxSpeed, addSpeed, shotType, special);
	}
	
	/*
	createShot42に発射地点も指定できるもの
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ xPos         : 弾の発射されるX座標
	/ yPos         :
	/ zPos         :
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ firstSpeed   : 弾の初期速度。
	/ maxSpeed     : 弾の最高速度
	/ addSpeed     : 弾の加速度
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot43(EntityLivingBase user, double xPos, double yPos, double zPos, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int shotType, int special)
	{
		int shotForm = shotType / 8;//形状は8の倍数で変化する

		return createShot44(user, xPos, yPos, zPos, angleXZ, angleY,
						firstSpeed, maxSpeed, addSpeed, DAMAGE[shotForm], shotType, SIZE[shotForm], 120, special);
	}
	
	/*
	createShot43に弾の大きさやダメージ、消滅時間までも指定できるもの（重力以外は指定できる）
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ xPos         : 弾の発射されるX座標
	/ yPos         :
	/ zPos         :
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ firstSpeed   : 弾の初期速度。
	/ maxSpeed     : 弾の最高速度
	/ addSpeed     : 弾の加速度
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	public static final EntityTHShot createShot44(EntityLivingBase user, double xPos, double yPos, double zPos, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, 
		int damage, int shotType, float size, int endTime, int special)
	{
		return createShotBaseB(user, user, xPos, yPos, zPos, angleXZ, angleY,
						firstSpeed, maxSpeed, addSpeed, 0.0D, 0.0D, 0.0D, damage, shotType, size, endTime, special);
	}
	
	/*
	使用者が発射地点になる一般的な弾を生成の特殊弾を指定できるもの
	/ EntityLivingBase : 発射地点であり、発射主になる。
	/ xPos         : 弾の発射されるX座標
	/ yPos         :
	/ zPos         :
	/ angleXZ      : 水平方向の角度をDegree（度数）で指定。0F ～ 360Fみたいな感じで
	/ angleY       : 垂直方向の角度。同上
	/ firstSpeed   : 弾の初期速度。
	/ maxSpeed     : 弾の最高速度
	/ addSpeed     : 弾の加速度
	/ shotType     : 弾番号を指定。整数でも指定できるが、thShotLib.SMALL[thShotLib.RED] などと指定もできる。この場合、赤い小弾が発射される
	*/
	/*public static final void createShot43(EntityLivingBase user, double xPos, double yPos, double zPos, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int shotType, int special)
	{
		int shotForm = shotType / 8;//形状は8の倍数で変化する

		createShotBaseB(user, user, xPos, yPos, zPos, angleXZ, angleY,
						firstSpeed, maxSpeed, addSpeed, 0.0D, 0.0D, 0.0D, DAMAGE[shotForm], shotType, SIZE[shotForm], 120, special);
	}*/
	
	/*
	すべてのn-way弾の基礎となる処理
	user         : 弾の持ち主、当たった場合持ち主が当てたことになる。また、持ち主が死ぬと消滅したり、アイテム化する
	source       : 弾の発射元。あまり意味は無い
	xPos         : 弾のX座標の発射地点
	yPos         :
	zPos         :
	xVector      : 弾のX方向のベクトル値
	yVector      :
	zVector      :
	firstSpeed   : 弾の初期速度
	maxSpeed     : 弾の最高速度。これ以上は速くならない
	addSpeed     : 弾の加速度。毎フレームこの数値分速くなる
	xVectorG     : 弾のX軸方向の重力値。0.0Dで重力の影響を受けない
	yVectorG     :
	zVectorG     :
	damage       : 弾のダメージ
	form         : 弾の形状。小弾、大弾、星弾、光弾などはこれできまる。また色も決まる。thShotLib.BUTTERFLY[ thShotLib.YELLOW ]というように指定できる
	size         : 弾の大きさ。当たり判定、描画サイズに関わる
	endTime      : 弾の消滅時間。この時間になったら強制的に消滅する。懐中時計で止められている場合は時間は進まず停滞する
	special      : 弾の特殊な処理を指定する。普通の弾は0。FALL01とすれば落下弾になる
	way          : 弾数
	wideAngle    : 扇状の成す角度
	distance     : 発射地点からこの分だけ離して発射する
	*/
	public static final void createWideShotBaseA(EntityLivingBase user, Entity source, double xPos, double yPos, double zPos,
											double xVector, double yVector, double zVector, double firstSpeed, double maxSpeed, double addSpeed,
											double xVectorG, double yVectorG, double zVectorG,
											int damage, int form, float size, int endTime, int special, int way, float wideAngle, double distance)
	{
		float disXZ = (float)Math.sqrt( xVector * xVector + zVector * zVector );
    	float angleXZ = (float)Math.atan2(-xVector, zVector);
    	float angleY  = (float)Math.atan2(-yVector, disXZ);
		float baseYaw = angleXZ;
		float basePitch = angleY;
		double X1,X2,Z1,Z2,/*angleXZ = 0,angleY = 0,*/ vecX, vecY, vecZ;
		float angleXZ_rad, angleY_rad;
		Vec3 lookAt = Vec3.createVectorHelper(xVector, yVector, zVector);
		lookAt.rotateAroundY((float)Math.PI*2);
    	float f = 20F, angle, spanAngle;
    	angle = -radTrans(wideAngle / 2F);
		spanAngle = radTrans(wideAngle / (float)way); //あらゆる弾数でも360度飛ぶように、弾の間隔
		
		boolean randomColor = form - (form % 1000) == 1000;
		if(randomColor)
		{
			form -= 1000;
		}
		Random rand = new Random();
    	int color = form;
		for(int i = 0; i < way; i++)
    	{	
    		//飛ぶ方向を設定
			angleXZ = angle;//横の角度　0=正面　+1ごとに左にずれていき360で正面に戻る
			angleY = 0;//縦の角度　0=正面　+1ごとに上にずれていき360で正面に戻る
    		angleY_rad = radTrans(angleY);
    		
			X1 =  Math.sin(angleXZ) * Math.cos(baseYaw);
			Z1 =  Math.sin(angleXZ) * Math.sin(baseYaw);
			X2 =  Math.cos(angleXZ) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.sin(baseYaw);
			Z2 =  Math.cos(angleXZ) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.cos(baseYaw);
			
			//vecY = -Math.cos(angleXZ_rad) * Math.sin(radTrans(user.rotationPitch - angleY));//Y方向　上下
    		vecY = -Math.cos(angleXZ) * Math.sin(basePitch - angleY_rad);//Y方向　上下
			vecX =  Math.cos(angleXZ) * Math.cos(angleY_rad) * lookAt.xCoord + X1 - X2;//X方向　水平方向
			vecZ =  Math.cos(angleXZ) * Math.cos(angleY_rad) * lookAt.zCoord + Z1 + Z2;//Z方向　水平方向
   			
			if(randomColor)
			{
				color = form + rand.nextInt(7);
			}
    		createShotBaseA(user, source, xPos + vecX * distance, yPos + vecY * distance, zPos + vecZ * distance, vecX, vecY, vecZ, firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG, damage, color, size, endTime, special);
			
    		angle += spanAngle;
    		//basePitch -= pitchSpan * (float)Math.cos(basePitch);
   		}
	}
	
	/*
	すべてのn-way弾の基礎となる処理
	user         : 弾の持ち主、当たった場合持ち主が当てたことになる。また、持ち主が死ぬと消滅したり、アイテム化する
	source       : 弾の発射元。あまり意味は無い
	xPos         : 弾のX座標の発射地点
	yPos         :
	zPos         :
	xVector      : 弾のX方向のベクトル値
	yVector      :
	zVector      :
	firstSpeed   : 弾の初期速度
	maxSpeed     : 弾の最高速度。これ以上は速くならない
	addSpeed     : 弾の加速度。毎フレームこの数値分速くなる
	xVectorG     : 弾のX軸方向の重力値。0.0Dで重力の影響を受けない
	yVectorG     :
	zVectorG     :
	damage       : 弾のダメージ
	form         : 弾の形状。小弾、大弾、星弾、光弾などはこれできまる。また色も決まる。thShotLib.BUTTERFLY[ thShotLib.YELLOW ]というように指定できる
	size         : 弾の大きさ。当たり判定、描画サイズに関わる
	endTime      : 弾の消滅時間。この時間になったら強制的に消滅する。懐中時計で止められている場合は時間は進まず停滞する
	special      : 弾の特殊な処理を指定する。普通の弾は0。FALL01とすれば落下弾になる
	way          : 弾数
	wideAngle    : 扇状の成す角度
	distance     : 発射地点からこの分だけ離して発射する
	*/
	public static final void createWideShotBaseB(EntityLivingBase user, Entity source, double xPos, double yPos, double zPos,
											float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed,
											double xVectorG, double yVectorG, double zVectorG,
											int damage, int form, float size, int endTime, int special, int way, float wideAngle, double distance)
	{
		float angleXZ_rad = radTrans(angleXZ);//水平角度をラジアンに変更
		float angleY_rad = radTrans(angleY);//垂直角度をラジアンに変更
		double xVector = -Math.sin(angleXZ_rad) * Math.cos(angleY_rad);
		double yVector = -Math.sin(angleY_rad);
		double zVector =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad);
		
		createWideShotBaseA(user, source, xPos, yPos, zPos, xVector, yVector, zVector, firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG,
				damage, form, size, endTime, special, way, wideAngle, distance);
	}
	
	/*
	最も一般的な扇状弾
	way       : 弾数
	wideAngle : 扇の広がる角度
	*/
	/*public static final void createWideShot01(EntityLivingBase EntityLivingBase, float angleXZ, float angleY, double speed, int shotType, int way, float wideAngle)
	{
		float angle = thKaguyaLib.radTrans(angleXZ - wideAngle / 2F);//初期角度
		float span = thKaguyaLib.radTrans(wideAngle / (float)way);//弾と弾の間の角度
		for(int i = 0; i < way; i++)
		{
			angle += span;
		}
		
	}*/
	public static final void createWideShot01(EntityLivingBase user, float angleXZ, float angleY, double speed, int shotType, int way, float wideAngle)
	{
		int shotForm = shotType;
		if(shotForm >= 1000)
		{
			shotForm %= 1000;
		}
		shotForm = shotForm / 8;//形状は8の倍数で変化する
		createWideShotBaseB(user, user, user.posX, user.posY, user.posZ, angleXZ, angleY, speed, speed, 1.0D, 0.0D, 0.0D, 0.0D,
				DAMAGE[shotForm], shotType, SIZE[shotForm], 120, 0, way, wideAngle, 0.0D);
	}
	
	/*
	すべての全方位弾の基礎となる処理
	user         : 弾の持ち主、当たった場合持ち主が当てたことになる。また、持ち主が死ぬと消滅したり、アイテム化する
	source       : 弾の発射元。あまり意味は無い
	xPos         : 弾のX座標の発射地点
	yPos         :
	zPos         :
	xVector      : 弾のX方向のベクトル値
	yVector      :
	zVector      :
	firstSpeed   : 弾の初期速度
	maxSpeed     : 弾の最高速度。これ以上は速くならない
	addSpeed     : 弾の加速度。毎フレームこの数値分速くなる
	xVectorG     : 弾のX軸方向の重力値。0.0Dで重力の影響を受けない
	yVectorG     :
	zVectorG     :
	damage       : 弾のダメージ
	form         : 弾の形状。小弾、大弾、星弾、光弾などはこれできまる。また色も決まる。thShotLib.BUTTERFLY[ thShotLib.YELLOW ]というように指定できる
	size         : 弾の大きさ。当たり判定、描画サイズに関わる
	endTime      : 弾の消滅時間。この時間になったら強制的に消滅する。懐中時計で止められている場合は時間は進まず停滞する
	special      : 弾の特殊な処理を指定する。普通の弾は0。FALL01とすれば落下弾になる
	way          : 全方位の弾数
	distance     : 発射地点からこの分だけ離して発射する
	*/
	public static final void createCircleShotBaseA(EntityLivingBase user, Entity source, double xPos, double yPos, double zPos,
											double xVector, double yVector, double zVector, double firstSpeed, double maxSpeed, double addSpeed,
											double xVectorG, double yVectorG, double zVectorG,
											int damage, int form, float size, int endTime, int special, int way, double distance)
	{
		float disXZ = (float)Math.sqrt( xVector * xVector + zVector * zVector );
    	float angleXZ = (float)Math.atan2(-xVector, zVector);
    	float angleY  = (float)Math.atan2(-yVector, disXZ);
		//float baseYaw = radTrans(user.rotationYaw);
		//float basePitch = radTrans(user.rotationPitch);
		float baseYaw = angleXZ;
		float basePitch = angleY;
		double X1,X2,Z1,Z2,/*angleXZ = 0,angleY = 0,*/ vecX, vecY, vecZ;
		float angleXZ_rad, angleY_rad;
		Vec3 lookAt = Vec3.createVectorHelper(xVector, yVector, zVector);
		//Vec3 lookAt = user.getLookVec();
		lookAt.rotateAroundY((float)Math.PI*2);
    	float f = 20F, angle = 0F, spanAngle;
		spanAngle = 2.0F * (float)Math.PI / (float)way; //あらゆる弾数でも360度飛ぶように、弾の間隔
		//pitchSpan = basePitch / ((float)way / 4.0F);
    	
		//float rotationYaw = atan2();
		for(int i = 0; i < way; i++)
    	{	
    		//飛ぶ方向を設定
			angleXZ = angle;//横の角度　0=正面　+1ごとに左にずれていき360で正面に戻る
			angleY = 0;//縦の角度　0=正面　+1ごとに上にずれていき360で正面に戻る
    		//angleXZ_rad = radTrans(angleXZ);
    		angleY_rad = radTrans(angleY);
    		
			X1 =  Math.sin(angleXZ) * Math.cos(baseYaw);
			Z1 =  Math.sin(angleXZ) * Math.sin(baseYaw);
			X2 =  Math.cos(angleXZ) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.sin(baseYaw);
			Z2 =  Math.cos(angleXZ) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.cos(baseYaw);
			
			//vecY = -Math.cos(angleXZ_rad) * Math.sin(radTrans(user.rotationPitch - angleY));//Y方向　上下
    		vecY = -Math.cos(angleXZ) * Math.sin(basePitch - angleY_rad);//Y方向　上下
			vecX =  Math.cos(angleXZ) * Math.cos(angleY_rad) * lookAt.xCoord + X1 - X2;//X方向　水平方向
			vecZ =  Math.cos(angleXZ) * Math.cos(angleY_rad) * lookAt.zCoord + Z1 + Z2;//Z方向　水平方向
   			
    		createShotBaseA(user, source, xPos + vecX * distance, yPos + vecY * distance, zPos + vecZ * distance, vecX, vecY, vecZ, firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG, damage, form, size, endTime, special);
			
    		angle += spanAngle;
    		//basePitch -= pitchSpan * (float)Math.cos(basePitch);
   		}
	}
	
	
	public static final void createCircleShotBaseB(EntityLivingBase user, Entity source, double xPos, double yPos, double zPos,
											float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed,
											double xVectorG, double yVectorG, double zVectorG,
											int damage, int form, float size, int endTime, int special, int way, double distance)
	{
		float angleXZ_rad = radTrans(angleXZ);//水平角度をラジアンに変更
		float angleY_rad = radTrans(angleY);//垂直角度をラジアンに変更
		double xVector = -Math.sin(angleXZ_rad) * Math.cos(angleY_rad);
		double yVector = -Math.sin(angleY_rad);
		double zVector =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad);
		
		createCircleShotBaseA(user, source, xPos, yPos, zPos, xVector, yVector, zVector, firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG, damage, form, size, endTime, special, way, distance);
	}
	
	/*
	最も一般的な全方位弾を発射する
	/ user : 使用者
	/ angleXZ  : 水平方向の角度
	/ angleY   : 垂直方向の角度
	/ speed    : 弾の速さ
	/ shotType : 弾の形状及び色
	/ way      : 全方位の弾数
	*/
	public static final void createCircleShot01(EntityLivingBase user, float angleXZ, float angleY, double speed, int shotType, int way)
	{
		int shotForm = shotType / 8;//形状は8の倍数で変化する
		createCircleShotBaseB(user, user, user.posX, user.posY, user.posZ, angleXZ, angleY, speed, speed, 1.0D, 0.0D, 0.0, 0.0D, DAMAGE[shotForm], shotType, SIZE[shotForm], 120, 0, way, (double)SIZE[shotForm]);
	}
	
	/*
	最も一般的な全方位弾を発射する
	/ vectorX  :  X方向のベクトル
	/ vectorY  :  Y方向のベクトル
	/ vectorZ  :  Z方向のベクトル
	*/
	public static final void createCircleShot11(EntityLivingBase EntityLivingBase, double vectorX, double vectorY, double vectorZ, double speed, int shotType, int way)
	{
		double X1,X2,Z1,Z2,angleXZ = 0,angleY = 0, vecX, vecY, vecZ;
		float angleXZ_rad, angleY_rad;
		Vec3 lookAt = Vec3.createVectorHelper(vectorX, vectorY, vectorZ);
		lookAt.rotateAroundY((float)Math.PI*2);
    	float f = 20F, angle = 0F, spanAngle;
    	spanAngle = 360F/(float)way; //あらゆる弾数でも360度飛ぶように、弾の間隔
		EntityTHShot entityTHShot;
		int shotForm = shotType / 8;//形状は8の倍数で変化する
    	
		float baseYaw = radTrans(EntityLivingBase.rotationYaw);
		float basePitch = radTrans(EntityLivingBase.rotationPitch);
		for(int i = 0; i < way; i++)
    	{	
    		//飛ぶ方向を設定
			angleXZ = angle;//横の角度　0=正面　+1ごとに左にずれていき360で正面に戻る
			angleY = 0;//縦の角度　0=正面　+1ごとに上にずれていき360で正面に戻る
    		angleXZ_rad = radTrans(angleXZ);
    		angleY_rad = radTrans(angleY);
    		
			X1 =  Math.sin(angleXZ_rad) * Math.cos(baseYaw);
			Z1 =  Math.sin(angleXZ_rad) * Math.sin(baseYaw);
			X2 =  Math.cos(angleXZ_rad) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.sin(baseYaw);
			Z2 =  Math.cos(angleXZ_rad) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.cos(baseYaw);
			
			vecY = -Math.cos(angleXZ_rad) * Math.sin(radTrans(EntityLivingBase.rotationPitch - angleY));//Y方向　上下
			vecX =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad) * lookAt.xCoord + X1 - X2;//X方向　水平方向
			vecZ =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad) * lookAt.zCoord + Z1 + Z2;//Z方向　水平方向
    		
    		entityTHShot = new EntityTHShot(EntityLivingBase.worldObj, EntityLivingBase, vecX, vecY, vecZ, speed, DAMAGE[shotForm], shotType, SIZE[shotForm]);//蝶弾の配列に方向、色などを代入
   			
			if(!EntityLivingBase.worldObj.isRemote)
			{
				//弾を生成する
				EntityLivingBase.worldObj.spawnEntityInWorld(entityTHShot);
			}
			
    		angle += spanAngle;
   		}
	}
	
	/*
	最も一般的な全方位弾を発射する
	/ vectorX  :  X方向のベクトル
	/ vectorY  :  Y方向のベクトル
	/ vectorZ  :  Z方向のベクトル
	*/
	public static final void createCircleShot21(EntityLivingBase EntityLivingBase, float base_angleXZ, float base_angleY, double vectorX, double vectorY, double vectorZ, 
		double speed, double maxSpeed, double accelerate, int shotType, int damage, int special, int way)
	{
		double X1,X2,Z1,Z2,angleXZ = 0,angleY = 0, vecX, vecY, vecZ;
		float angleXZ_rad, angleY_rad;
		Vec3 lookAt = Vec3.createVectorHelper(vectorX, vectorY, vectorZ);
		lookAt.rotateAroundY((float)Math.PI*2);
    	float f = 20F, angle = 0F, spanAngle;
    	spanAngle = 360F/(float)way; //あらゆる弾数でも360度飛ぶように、弾の間隔
		EntityTHShot entityTHShot;
		int shotForm = shotType / 8;//形状は8の倍数で変化する
    	
		float baseYaw = radTrans(base_angleXZ);
		float basePitch = radTrans(base_angleY);
		for(int i = 0; i < way; i++)
    	{	
    		//飛ぶ方向を設定
			angleXZ = angle;//横の角度　0=正面　+1ごとに左にずれていき360で正面に戻る
			angleY = 0;//縦の角度　0=正面　+1ごとに上にずれていき360で正面に戻る
    		angleXZ_rad = radTrans(angleXZ);
    		angleY_rad = radTrans(angleY);
    		
			X1 =  Math.sin(angleXZ_rad) * Math.cos(baseYaw);
			Z1 =  Math.sin(angleXZ_rad) * Math.sin(baseYaw);
			X2 =  Math.cos(angleXZ_rad) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.sin(baseYaw);
			Z2 =  Math.cos(angleXZ_rad) * Math.sin(angleY_rad) * Math.sin(basePitch) * Math.cos(baseYaw);
			
			vecY = -Math.cos(angleXZ_rad) * Math.sin(radTrans(base_angleY - angleY));//Y方向　上下
			vecX =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad) * lookAt.xCoord + X1 - X2;//X方向　水平方向
			vecZ =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad) * lookAt.zCoord + Z1 + Z2;//Z方向　水平方向
    		
    		//entityTHShot = new EntityTHShot(EntityLivingBase.worldObj, EntityLivingBase, vecX, vecY, vecZ, speed, DAMAGE[shotForm], shotType, SIZE[shotForm]);//蝶弾の配列に方向、色などを代入
   			entityTHShot = new EntityTHShot(EntityLivingBase.worldObj, EntityLivingBase, EntityLivingBase, 
			EntityLivingBase.posX, EntityLivingBase.posY, EntityLivingBase.posZ,
			vecX, vecY, vecZ,
			speed, maxSpeed, accelerate,
			0.0D, 0.0D, 0.0D, damage/*DAMAGE[shotForm]*/, shotType, SIZE[shotForm], 120, special);
    		
    		
			if(!EntityLivingBase.worldObj.isRemote)
			{
				//弾を生成する
				EntityLivingBase.worldObj.spawnEntityInWorld(entityTHShot);
			}
			
    		angle += spanAngle;
   		}
	}
	
	/*
	移動レーザー発射の基礎処理
	基本的に弾発射の基礎処理と同じ
	width  : レーザーの太さ。弾のsizeに該当。当たり判定の太さ
	length : レーザーの長さ。当たり判定も長さの分だけある
	*/
	public static final void createLaserABaseA(	EntityLivingBase user, Entity source, double xPos, double yPos, double zPos, double xVector, double yVector, double zVector,
												double firstSpeed, double maxSpeed, double addSpeed, double xVectorG, double yVectorG, double zVectorG,
												int damage, int color, float width, int endTime, int special, double length)
	{
		EntityTHLaser laser = new EntityTHLaser(source.worldObj, user, source, xPos, yPos, zPos, xVector, yVector, zVector, 
												firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG,
												damage, color, width, endTime, special, length);
		if(!source.worldObj.isRemote)
		{
			//レーザーを生成する
			source.worldObj.spawnEntityInWorld(laser);
		}
	}
	
	/*
	移動レーザー発射の基礎処理の角度指定版
	基本的に弾発射の基礎処理と同じ
	width  : レーザーの太さ。弾のsizeに該当。当たり判定の太さ
	length : レーザーの長さ。当たり判定も長さの分だけある
	*/
	public static final void createLaserABaseB(	EntityLivingBase user, Entity source, double xPos, double yPos, double zPos, float angleXZ, float angleY,
												double firstSpeed, double maxSpeed, double addSpeed, double xVectorG, double yVectorG, double zVectorG,
												int damage, int color, float width, int endTime, int special, double length)
	{
		float angleXZ_rad = radTrans(angleXZ);//水平角度をラジアンに変更
		float angleY_rad = radTrans(angleY);//垂直角度をラジアンに変更
		double xVector = -Math.sin(angleXZ_rad) * Math.cos(angleY_rad);
		double yVector = -Math.sin(angleY_rad);
		double zVector =  Math.cos(angleXZ_rad) * Math.cos(angleY_rad);
		
		createLaserABaseA(	user, source, xPos, yPos, zPos, xVector, yVector, zVector, 
							firstSpeed, maxSpeed, addSpeed, xVectorG, yVectorG, zVectorG,
							damage, color, width, endTime, special, length);
	}
	
	/*
	最も基本的な一般的な移動レーザーを生成
	user : レーザーの使用者
	angleXZ : レーザーの水平方向の角度
	angleY  : レーザーの水平方向の角度
	speed   : レーザーの速度
	damage  : レーザーのダメージ
	color   : レーザーの色
	width   : レーザーの太さ
	length  : レーザーの長さ
	*/
	public static final void createLaserA01(EntityLivingBase user, float angleXZ, float angleY, double speed, int damage, int color, float width, double length)
	{
		createLaserA02(	user, angleXZ, angleY, speed, speed, 1.0D, damage, color, width, length);
	}
	
	/*
	加速度を指定できる移動レーザーを生成
	user       : レーザーの使用者
	angleXZ    : レーザーの水平方向の角度
	angleY     : レーザーの水平方向の角度
	firstSpeed : レーザーの初期速度
	maxSpeed   : レーザーの最高速度
	addSpeed   ; レーザーの加速度
	damage     : レーザーのダメージ
	color      : レーザーの色
	width      : レーザーの太さ
	length     : レーザーの長さ
	*/
	public static final void createLaserA02(EntityLivingBase user, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int damage, int color, float width, double length)
	{
		createLaserA03(	user, user.posX, user.posY, user.posZ, angleXZ, angleY, firstSpeed, maxSpeed, addSpeed, damage, color, width, length);
	}
	
	/*
	加速度と発射地点を指定できる移動レーザーを生成
	user       : レーザーの使用者
	xPos       : レーザーの発射地点のX座標
	yPos       :
	zPos       :
	angleXZ    : レーザーの水平方向の角度
	angleY     : レーザーの水平方向の角度
	firstSpeed : レーザーの初期速度
	maxSpeed   : レーザーの最高速度
	addSpeed   ; レーザーの加速度
	damage     : レーザーのダメージ
	color      : レーザーの色
	width      : レーザーの太さ
	length     : レーザーの長さ
	*/
	public static final void createLaserA03(EntityLivingBase user, double xPos, double yPos, double zPos, float angleXZ, float angleY, double firstSpeed, double maxSpeed, double addSpeed, int damage, int color, float width, double length)
	{
		createLaserABaseB(	user, user, xPos, yPos, zPos, angleXZ, angleY, firstSpeed, maxSpeed, addSpeed, 0.0D, 0.0D, 0.0D,
							damage, color, width, 120, 0, length);
	}
	
	//DegをRadに変換する
	public static final float radTrans(float deg)
	{
		return (float)((double)deg / 180.0D * Math.PI);
	}
	
	//DegをRadに変換する
	public static final float radTrans(double deg)
	{
		return (float)(deg / 180.0D * Math.PI);
	}
	
	//３Dベクトルを返す
}